## INTRODUCTION

The C2PA Sign AWS KMS module works with c2pa_sign to use a remote signer (in this case Amazon's Key Management Service) to sign the maniufests being embedded in a media file..

The primary use case for this module is:

- You want to use the C2Pa Sign module to add manifests and sign them but you don't want to have a software cert on your system. Instead you are using Amazon's AWS services and want their Key Management SErvice (KMS) to manage your keys

## REQUIREMENTS

- add the aws/aws-sdk-php library to your code base
- Create a new Customer Managed Key in AWS KMS
- Create a Certificate Signing Request from the key (I suggest using the process described here: https://github.com/g-a-d/aws-kms-sign-csr/tree/master)
- Place the PEM file for the signed certificate in your certificate directory use by C2PA Sign module. This certificate will automatically be detected and matched without you KMS key (the fingerprint of the certificate public key will be matched with the fingerprint of the KMS public key).
- Drush

## C2PATOOL
Version [0.7.0 of c2patool (Released Nov 22, 2023)](https://github.com/contentauth/c2patool/releases/tag/v0.7.0) doesn't include the ability to use a remote signer. This feature was added in [Pull Request 169 (MINOR) allow clients to sign with a process outside of c2patool](https://github.com/contentauth/c2patool/pull/169) and was merged in March 2024. As of this writing (April 17, 2024) a new release with this feature has not occurred so it will be necessary to compile your own release from the __main__ branch or from a prerelease tag >=0.8.0. If c2patool is put in a loction in the __$PATH__ it will be automatically discovered.

## Drush
C2patool using a remote signer takes a command line option ```--signer-path``` which expects for the location of a command line executable. The message to be signed is passed in to this executabe through __stdin__ and the signature is passed back through __stdout__. No command line arguments are passed through from the c2patool command line to the signer executable.

It was decided that the easiest way to provide a commnd line executable which can access the configuration to get the AWS KMS key information (to know what key to use for the signing) was to use a Drush command ```c2pa_sign_aws_kms:sign (c2pa_kms_sign)```. Unfortunately c2patool does not pass any arguments throuh to the ```--signer-path``` which makes Drush harder to use. A [bash script](https://git.drupalcode.org/project/c2pa_sign_aws_kms/-/blob/1.0.x/c2pa_sign_aws_kms.sh?ref_type=heads) is used to call the Drush command and pass through the additonal arguments (like the signing algorithm) to be used. Since the signing occurs through a Drush command the location of the Drush executable needs to be easily knowable. The easiest way to do this is to ensure that the Drush executable is in the __$PATH__ where is can be easily found. If you cannot plade Drush in the path you will need to edit the [bash script](https://git.drupalcode.org/project/c2pa_sign_aws_kms/-/blob/1.0.x/c2pa_sign_aws_kms.sh?ref_type=heads) to include the location od Drush.

## INSTALLATION

Install as you would normally install a contributed Drupal module.
See: https://www.drupal.org/node/895232 for further information.

## CONFIGURATION
- Enable c2pa_sign and c2pa_sign_aws_kms modules
- place your AWS credentials (for a use with permissions to use the cutomer managed key to sign data) on your server either in an environment variable which will be loaded when  the Apache user executes a Drush command or in the ~/.aws/credentials file for the user executing PHP from the browser/commandline user.
```text
[default]
aws_access_key_id = <key-id>
aws_secret_access_key = <access-key>
```
- Place your AWS configuration in file ~/.aws/config, the region to use for AWS SDK calls is important.
```text
[default]
region = us-gov-west-1
output = json
```
- Configure c2pa_sign to use the KMS managed key (choose from the list of key aliases)
- place your signed ertificate in the C2PA Sign module's configured certificate directory.

## MAINTAINERS

Current maintainers for Drupal 10:

- James Glasgow (jrglasgow) - https://www.drupal.org/u/jrglasgow
